@@ -1,3 +1,7 @@
+.onLoad <- function(...) {
+  run_on_load()
+}
+
 print_plot.patchwork <- function(p, title = '') {
   if (is.null(p$patches$annotation$title)) {
     p <- p + plot_annotation(title = title)
@@ -5,11 +9,6 @@ print_plot.patchwork <- function(p, title = '') {
   print(p)
 }
 
-.onLoad <- function(...) {
-  register_s3_method("vdiffr", "print_plot", "patchwork")
-  invisible()
-}
-
 register_s3_method <- function(pkg, generic, class, fun = NULL) {
   check_string(pkg)
   check_string(generic)
@@ -33,3 +32,6 @@ register_s3_method <- function(pkg, generic, class, fun = NULL) {
     }
   )
 }
+on_load(
+  register_s3_method("vdiffr", "print_plot", "patchwork")
+)
